@@ -11,7 +11,7 @@
    So that transport compiled against an older version of this
    header will no longer load in a module that assumes a newer
    version. */
-#define DRBD_TRANSPORT_API_VERSION 16
+#define DRBD_TRANSPORT_API_VERSION 18
 
 /* MSG_MSG_DONTROUTE and MSG_PROBE are not used by DRBD. I.e.
    we can reuse these flags for our purposes */
@@ -41,6 +41,8 @@
 	tr_printk(KERN_ERR, transport, fmt, ## args)
 #define tr_warn(transport, fmt, args...) \
 	tr_printk(KERN_WARNING, transport, fmt, ## args)
+#define tr_notice(transport, fmt, args...) \
+	tr_printk(KERN_NOTICE, transport, fmt, ## args)
 #define tr_info(transport, fmt, args...) \
 	tr_printk(KERN_INFO, transport, fmt, ## args)
 
@@ -78,6 +80,11 @@ enum drbd_tr_free_op {
 	DESTROY_TRANSPORT
 };
 
+enum drbd_tr_event {
+	CLOSED_BY_PEER,
+	TIMEOUT,
+};
+
 struct drbd_listener;
 
 /* A transport might wrap its own data structure around this. Having
@@ -129,6 +136,11 @@ struct drbd_page_chain_head {
 	unsigned int nr_pages;
 };
 
+struct drbd_const_buffer {
+	const u8 *buffer;
+	unsigned int avail;
+};
+
 struct drbd_transport_ops {
 	void (*free)(struct drbd_transport *, enum drbd_tr_free_op free_op);
 	int (*connect)(struct drbd_transport *);
@@ -181,6 +193,7 @@ struct drbd_transport_ops {
 	int (*recv_pages)(struct drbd_transport *, struct drbd_page_chain_head *, size_t size);
 
 	void (*stats)(struct drbd_transport *, struct drbd_transport_stats *stats);
+	void (*net_conf_change)(struct drbd_transport *, struct net_conf *new_net_conf);
 	void (*set_rcvtimeo)(struct drbd_transport *, enum drbd_stream, long timeout);
 	long (*get_rcvtimeo)(struct drbd_transport *, enum drbd_stream);
 	int (*send_page)(struct drbd_transport *, enum drbd_stream, struct page *,
@@ -215,6 +228,8 @@ struct drbd_listener {
 	spinlock_t waiters_lock;
 	int pending_accepts;
 	struct sockaddr_storage listen_addr;
+	struct completion ready;
+	int err;
 	void (*destroy)(struct drbd_listener *);
 };
 
@@ -236,11 +251,13 @@ extern void drbd_put_listener(struct drbd_path *path);
 extern struct drbd_path *drbd_find_path_by_addr(struct drbd_listener *, struct sockaddr_storage *);
 extern bool drbd_stream_send_timed_out(struct drbd_transport *transport, enum drbd_stream stream);
 extern bool drbd_should_abort_listening(struct drbd_transport *transport);
-extern void drbd_path_event(struct drbd_transport *transport, struct drbd_path *path);
+extern void drbd_path_event(struct drbd_transport *transport, struct drbd_path *path, bool destroyed);
 
 /* drbd_receiver.c*/
 extern struct page *drbd_alloc_pages(struct drbd_transport *, unsigned int, gfp_t);
 extern void drbd_free_pages(struct drbd_transport *transport, struct page *page, int is_net);
+extern void drbd_control_data_ready(struct drbd_transport *transport, struct drbd_const_buffer *pool);
+extern void drbd_control_event(struct drbd_transport *transport, enum drbd_tr_event);
 
 static inline void drbd_alloc_page_chain(struct drbd_transport *t,
 	struct drbd_page_chain_head *chain, unsigned int nr, gfp_t gfp_flags)
