@@ -153,7 +153,7 @@ struct windrbd_usermode_helper {
 		 * cmd<0>arg1<0>arg2<0>...argn<0>env1<0>env2<0> ... envn<0>
 		 * the above members determine how many args/how many envs.
 		 */
-	char data[0];
+	char data[];
 };
 
 struct windrbd_usermode_helper_return_value {
@@ -280,4 +280,31 @@ struct windrbd_minor_mount_point {
 
 #define IOCTL_WINDRBD_ROOT_CREATE_RESOURCE_FROM_URL CTL_CODE(WINDRBD_ROOT_DEVICE_TYPE, 14, METHOD_BUFFERED, FILE_ANY_ACCESS)
 
+/* Set the WinDRBD config key
+ * Input: The config key as a hex string
+ * Output: none
+ *
+ * Sets the key to lock writable DRBD commands
+ */
+
+#define IOCTL_WINDRBD_ROOT_SET_CONFIG_KEY CTL_CODE(WINDRBD_ROOT_DEVICE_TYPE, 15, METHOD_BUFFERED, FILE_ANY_ACCESS)
+
+/* Set the event log log level
+ * Input: A signed 32-bit value (0=emerg, 1=alert, ...)
+ * Output: none
+ *
+ * Sets the threshold value for messages that go into the Windows event log.
+ */
+
+#define IOCTL_WINDRBD_ROOT_SET_EVENT_LOG_LEVEL CTL_CODE(WINDRBD_ROOT_DEVICE_TYPE, 16, METHOD_BUFFERED, FILE_ANY_ACCESS)
+
+/* Report if config key is set (and required for manipulating resources)
+ * Input: none
+ * Output: an int: 1 .. system is locked 0 .. system is not locked
+ *
+ * Reports if config key is set
+ */
+
+#define IOCTL_WINDRBD_ROOT_GET_LOCK_DOWN_STATE CTL_CODE(WINDRBD_ROOT_DEVICE_TYPE, 17, METHOD_BUFFERED, FILE_ANY_ACCESS)
+
 #endif
