@@ -89,6 +89,8 @@ int genl_join_mc_group(struct genl_sock *s, const char *name)
         if (DeviceIoControl(s->s_handle, IOCTL_WINDRBD_ROOT_JOIN_MC_GROUP, (void*) &m, sizeof(m), NULL, 0, &unused, NULL) == 0) {
 	        err = GetLastError();
 		printf("DeviceIoControl() failed, error is %d\n", err);
+		if (err == ERROR_ACCESS_DENIED) /* 5 */
+			printf("(are you running with Administrator privileges?)\n");
 		return -1;
 	}
 	return 0;
@@ -123,6 +125,8 @@ static int do_send(struct genl_sock *s, const void *buf, int len)
         if (DeviceIoControl(s->s_handle, IOCTL_WINDRBD_ROOT_SEND_NL_PACKET, (void*) buf, len, NULL, 0, &unused, NULL) == 0) {
 	        err = GetLastError();
 		printf("DeviceIoControl() failed, error is %d\n", err);
+		if (err == ERROR_ACCESS_DENIED) /* 5 */
+			printf("(are you running with Administrator privileges?)\n");
 		return -1;
 	}
 	return 0;
@@ -169,6 +173,8 @@ int genl_recv_msgs(struct genl_sock *s, struct iovec *iov, char **err_desc, int
 	        if (DeviceIoControl(s->s_handle, IOCTL_WINDRBD_ROOT_RECEIVE_NL_PACKET, &p, sizeof(p), iov->iov_base, iov->iov_len, &size, NULL) == 0) {
 		        err = GetLastError();
 			printf("DeviceIoControl() failed, error is %d\n", err);
+			if (err == ERROR_ACCESS_DENIED) /* 5 */
+				printf("(are you running with Administrator privileges?)\n");
 			if (err_desc)
 				*err_desc = "ioctl error";
 			return -1;
