@@ -811,6 +811,11 @@ const char *on_no_data_map[] = {
 #define on_no_quorum_map on_no_data_map
 /* ONQ_XX == OND_XX */
 
+const char *on_susp_primary_outdated_map[] = {
+	[SPO_DISCONNECT] = "disconnect",
+	[SPO_FORCE_SECONDARY] = "force-secondary",
+};
+
 const char *on_congestion_map[] = {
 	[OC_BLOCK] = "block",
 	[OC_PULL_AHEAD] = "pull-ahead",
@@ -921,7 +926,15 @@ struct context_def primary_cmd_ctx = {
 	NLA_POLICY(set_role_parms),
 	.nla_type = DRBD_NLA_SET_ROLE_PARMS,
 	.fields = {
-		{ "force", FLAG(assume_uptodate) },
+		{ "force", FLAG(force) },
+		{ } },
+};
+
+struct context_def secondary_cmd_ctx = {
+	NLA_POLICY(set_role_parms),
+	.nla_type = DRBD_NLA_SET_ROLE_PARMS,
+	.fields = {
+		{ "force", FLAG(force) },
 		{ } },
 };
 
@@ -1024,6 +1037,7 @@ struct context_def resource_options_ctx = {
 		{ "quorum", ENUM_NUM(quorum, QUORUM, 1, DRBD_PEERS_MAX) },
 		{ "on-no-quorum", ENUM(on_no_quorum, ON_NO_QUORUM) },
 		{ "quorum-minimum-redundancy", ENUM_NUM(quorum_min_redundancy, QUORUM, 1, DRBD_PEERS_MAX) },
+		{ "on-suspended-primary-outdated", ENUM(on_susp_primary_outdated, ON_SUSP_PRI_OUTD) },
 		{ } },
 };
 
@@ -1056,12 +1070,24 @@ struct context_def device_options_ctx = {
 		{ } },
 };
 
+#define INVALIDATE_OPTIONS									\
+		{ "sync-from-peer-node-id", NUMERIC(sync_from_peer_node_id, SYNC_FROM_NID) },	\
+		{ "reset-bitmap", BOOLEAN(reset_bitmap, INVALIDATE_RESET_BITMAP) },
+
 struct context_def invalidate_ctx = {
 	NLA_POLICY(invalidate_parms),
 	.nla_type = DRBD_NLA_INVALIDATE_PARMS,
 	.fields = {
-		{ "sync-from-peer-node-id", NUMERIC(sync_from_peer_node_id, SYNC_FROM_NID) },
-		{ "reset-bitmap", BOOLEAN(reset_bitmap, INVALIDATE_RESET_BITMAP) },
+		INVALIDATE_OPTIONS
+		{ } },
+};
+
+struct context_def invalidate_adm_ctx = {
+	NLA_POLICY(invalidate_parms),
+	.nla_type = DRBD_NLA_INVALIDATE_PARMS,
+	.fields = {
+		{ "force", .argument_is_optional = true },
+		INVALIDATE_OPTIONS
 		{ } },
 };
 
@@ -1098,7 +1124,7 @@ struct context_def create_md_ctx = {
 		{ } },
 };
 
-struct context_def dump_md_ctx = {
+struct context_def forceable_ctx = {
        .fields = {
 		{ .name = "force", .argument_is_optional = true },
 		{ } },
