@@ -19,6 +19,7 @@
 
 #include "config.h"
 #include "shared_main.h"
+#include "path.h"
 
 /* FIXME keep in sync with GENL_MAGIC_VERSION,
  * without including all the genl magic...
@@ -419,11 +420,10 @@ extern void expand_common(void);
 extern void global_validate_maybe_expand_die_if_invalid(int expand, enum pp_flags flags);
 extern struct d_option *new_opt(char *name, char *value);
 extern int hostname_in_list(const char *name, struct names *names);
-extern char *_names_to_str(char* buffer, struct names *names);
-extern char *_names_to_str_c(char* buffer, struct names *names, char c);
-#define NAMES_STR_SIZE 255
-#define names_to_str(N) _names_to_str(alloca(NAMES_STR_SIZE+1), N)
-#define names_to_str_c(N, C) _names_to_str_c(alloca(NAMES_STR_SIZE+1), N, C)
+extern char *_names_to_str_c(char* buffer, int buffer_size, struct names *names, char c);
+#define names_to_str_c_max(NAMES, C, MAX_SIZE) _names_to_str_c(alloca(MAX_SIZE), MAX_SIZE, NAMES, C)
+#define names_to_str(NAMES) names_to_str_c_max(NAMES, ' ', 4096)
+#define names_to_str_c(NAMES, C) names_to_str_c_max(NAMES, C, 4096)
 extern struct d_name *names_from_str(char* str);
 extern struct d_volume *volume_by_vnr(struct volumes *volumes, int vnr);
 extern void free_names(struct names *names);
@@ -445,6 +445,8 @@ int parse_proxy_options_section(struct d_proxy_info **proxy);
 struct peer_device *find_peer_device(struct connection *conn, int vnr);
 bool peer_diskless(struct peer_device *peer_device);
 
+const struct field_def *find_field(bool *no_prefix, const struct context_def *options_def, const char *name);
+
 extern char *config_file;
 extern char *config_save;
 extern int config_valid;
@@ -588,5 +590,15 @@ extern char *khelper_argv[];
 void print_platform_specific_versions(void);
 void assign_default_device(struct d_volume *vol);
 
+void generate_conf_file_locations(void);
+void free_conf_file_locations(void);
+
+const char *drbd_lib_dir(void);
+const char *node_id_file(void);
+const char *drbd_run_dir(void);
+const char *drbd_run_dir_with_slash(void);
+const char *drbd_bin_dir(void);
+const char *drbd_lock_dir(void);
+
 #endif
 
