@@ -80,7 +80,6 @@ struct option general_admopt[] = {
 	{"drbdmeta", required_argument, 0, 'm'},
 	{"drbd-proxy-ctl", required_argument, 0, 'p'},
 	{"sh-varname", required_argument, 0, 'n'},
-	{"peer", required_argument, 0, 'P'},
 	{"version", no_argument, 0, 'V'},
 	{"setup-option", required_argument, 0, 'W'},
 	{"help", no_argument, 0, 'h'},
@@ -174,8 +173,6 @@ char *drbd_proxy_ctl;
 char *sh_varname = NULL;
 struct names backend_options = STAILQ_HEAD_INITIALIZER(backend_options);
 
-char *connect_to_host = NULL;
-
 STAILQ_HEAD(deferred_cmds, deferred_cmd) deferred_cmds[__CFG_LAST];
 
 int adm_adjust_wp(const struct cfg_ctx *ctx)
@@ -342,8 +339,8 @@ static struct adm_cmd up_cmd = {"up", adm_up, ACF1_RESNAME_VERIFY_IPS };
 /*  */ struct adm_cmd res_options_cmd = {"resource-options", adm_resource, &resource_options_ctx, ACF1_RESNAME};
 static struct adm_cmd down_cmd = {"down", adm_drbdsetup, ACF1_RESNAME .takes_long = 1};
 static struct adm_cmd primary_cmd = {"primary", adm_drbdsetup, &primary_cmd_ctx, ACF1_RESNAME .takes_long = 1};
-static struct adm_cmd secondary_cmd = {"secondary", adm_drbdsetup, ACF1_RESNAME .takes_long = 1};
-static struct adm_cmd invalidate_cmd = {"invalidate", adm_invalidate, &invalidate_ctx, ACF1_MINOR_ONLY };
+static struct adm_cmd secondary_cmd = {"secondary", adm_drbdsetup, &secondary_cmd_ctx, ACF1_RESNAME .takes_long = 1};
+static struct adm_cmd invalidate_cmd = {"invalidate", adm_invalidate, &invalidate_adm_ctx, ACF1_MINOR_ONLY };
 static struct adm_cmd invalidate_remote_cmd = {"invalidate-remote", adm_drbdsetup, &invalidate_peer_ctx, ACF1_PEER_DEVICE .takes_long = 1};
 static struct adm_cmd outdate_cmd = {"outdate", adm_outdate, ACF1_DEFAULT};
 /*  */ struct adm_cmd resize_cmd = {"resize", adm_resize, &resize_cmd_ctx, ACF1_DEFAULT .disk_required = 1};
@@ -357,7 +354,7 @@ static struct adm_cmd wait_sync_cmd = {"wait-sync", adm_wait_c, ACF1_WAIT};
 static struct adm_cmd wait_ci_cmd = {"wait-con-int", adm_wait_ci, .show_in_usage = 1,.verify_ips = 1,};
 static struct adm_cmd role_cmd = {"role", adm_drbdsetup, ACF1_RESNAME};
 static struct adm_cmd cstate_cmd = {"cstate", adm_drbdsetup, ACF1_DISCONNECT};
-static struct adm_cmd dstate_cmd = {"dstate", adm_setup_and_meta, ACF1_MINOR_ONLY .disk_required = 0 };
+static struct adm_cmd dstate_cmd = {"dstate", adm_setup_and_meta, &forceable_ctx, ACF1_MINOR_ONLY .disk_required = 0 };
 static struct adm_cmd status_cmd = {"status", adm_drbdsetup, .show_in_usage = 1, .uc_dialog = 1, .backend_res_name=1};
 static struct adm_cmd peer_device_options_cmd = {"peer-device-options", adm_peer_device,
 						 &peer_device_options_ctx, ACF1_PEER_DEVICE};
@@ -365,18 +362,18 @@ static struct adm_cmd dump_cmd = {"dump", adm_dump, ACF1_DUMP};
 static struct adm_cmd dump_xml_cmd = {"dump-xml", adm_dump_xml, ACF1_DUMP};
 
 static struct adm_cmd create_md_cmd = {"create-md", adm_create_md, &create_md_ctx, ACF1_MINOR_ONLY };
-static struct adm_cmd show_gi_cmd = {"show-gi", adm_setup_and_meta, ACF1_PEER_DEVICE .disk_required = 1};
-static struct adm_cmd get_gi_cmd = {"get-gi", adm_setup_and_meta, ACF1_PEER_DEVICE .disk_required = 1};
-static struct adm_cmd dump_md_cmd = {"dump-md", adm_drbdmeta, &dump_md_ctx, ACF1_MINOR_ONLY };
-static struct adm_cmd wipe_md_cmd = {"wipe-md", adm_drbdmeta, ACF1_MINOR_ONLY };
-static struct adm_cmd apply_al_cmd = {"apply-al", adm_drbdmeta, ACF1_MINOR_ONLY };
-static struct adm_cmd forget_peer_cmd = {"forget-peer", adm_forget_peer, ACF1_DISCONNECT };
+static struct adm_cmd show_gi_cmd = {"show-gi", adm_setup_and_meta, &forceable_ctx, ACF1_PEER_DEVICE .disk_required = 1};
+static struct adm_cmd get_gi_cmd = {"get-gi", adm_setup_and_meta, &forceable_ctx, ACF1_PEER_DEVICE .disk_required = 1};
+static struct adm_cmd dump_md_cmd = {"dump-md", adm_drbdmeta, &forceable_ctx, ACF1_MINOR_ONLY };
+static struct adm_cmd wipe_md_cmd = {"wipe-md", adm_drbdmeta, &forceable_ctx, ACF1_MINOR_ONLY };
+static struct adm_cmd apply_al_cmd = {"apply-al", adm_drbdmeta, &forceable_ctx, ACF1_MINOR_ONLY };
+static struct adm_cmd forget_peer_cmd = {"forget-peer", adm_forget_peer, &forceable_ctx, ACF1_DISCONNECT };
 
 static struct adm_cmd hidden_cmd = {"hidden-commands", hidden_cmds,.show_in_usage = 1,};
 
 static struct adm_cmd sh_nop_cmd = {"sh-nop", sh_nop, ACF2_GEN_SHELL .uc_dialog = 1, .test_config = 1};
 static struct adm_cmd sh_resources_cmd = {"sh-resources", sh_resources, ACF2_GEN_SHELL};
-static struct adm_cmd sh_resource_cmd = {"sh-resource", sh_resource, ACF2_SH_RESNAME};
+static struct adm_cmd sh_resource_cmd = {"sh-resource", sh_resource, ACF2_SH_RESNAME .vol_id_optional = 1};
 static struct adm_cmd sh_mod_parms_cmd = {"sh-mod-parms", sh_mod_parms, ACF2_GEN_SHELL};
 static struct adm_cmd sh_dev_cmd = {"sh-dev", sh_dev, ACF2_SHELL};
 static struct adm_cmd sh_udev_cmd = {"sh-udev", sh_udev, .vol_id_required = 1, ACF2_HOOK};
@@ -384,7 +381,7 @@ static struct adm_cmd sh_minor_cmd = {"sh-minor", sh_minor, ACF2_SHELL};
 static struct adm_cmd sh_ll_dev_cmd = {"sh-ll-dev", sh_ll_dev, ACF2_SHELL .disk_required = 0};
 static struct adm_cmd sh_md_dev_cmd = {"sh-md-dev", sh_md_dev, ACF2_SHELL .disk_required = 1};
 static struct adm_cmd sh_md_idx_cmd = {"sh-md-idx", sh_md_idx, ACF2_SHELL .disk_required = 1};
-static struct adm_cmd sh_ip_cmd = {"sh-ip", sh_ip, ACF2_SHELL};
+static struct adm_cmd sh_ip_cmd = {"sh-ip", sh_ip, ACF2_SHELL .need_peer = 1, .iterate_paths = 1};
 static struct adm_cmd sh_lr_of_cmd = {"sh-lr-of", sh_lres, ACF2_SHELL};
 
 static struct adm_cmd proxy_up_cmd = {"proxy-up", adm_proxy_up, ACF2_PROXY};
@@ -905,7 +902,9 @@ static int sh_minor(const struct cfg_ctx *ctx)
 
 static int sh_ip(const struct cfg_ctx *ctx)
 {
-	printf("%s\n", ctx->res->me->address.addr);
+	struct path *path = ctx->path;
+
+	printf("%s\n", path->my_address->addr);
 	return 0;
 }
 
@@ -1578,9 +1577,16 @@ bool backing_device_configured(const struct cfg_ctx *ctx)
  * if it fails, try drbdmeta */
 static int adm_setup_and_meta(const struct cfg_ctx *ctx)
 {
+	struct cfg_ctx tmp_ctx = *ctx;
+	struct adm_cmd tmp_cmd = *tmp_ctx.cmd;
 	int rv;
 
-	rv = __adm_drbdsetup_silent(ctx);
+	/* forceable_ctx is only for drbdmeta... */
+	if (tmp_cmd.drbdsetup_ctx == &forceable_ctx)
+		tmp_cmd.drbdsetup_ctx = NULL;
+	tmp_ctx.cmd = &tmp_cmd;
+
+	rv = __adm_drbdsetup_silent(&tmp_ctx);
 
 	if (rv == 11 || rv == 17 || !backing_device_configured(ctx)) {
 		/* see drbdsetup.c, print_config_error():
@@ -1600,6 +1606,7 @@ static int adm_invalidate(const struct cfg_ctx *ctx)
 	static const struct adm_cmd invalidate_meta_cmd = {
 		"invalidate",
 		adm_drbdmeta,
+		&forceable_ctx,
 		ACF1_MINOR_ONLY
 	};
 
@@ -1625,6 +1632,7 @@ static int adm_forget_peer(const struct cfg_ctx *ctx)
 	static const struct adm_cmd forget_peer_meta_cmd = {
 		"forget-peer",
 		adm_drbdmeta,
+		&forceable_ctx,
 		ACF1_PEER_DEVICE .disk_required = 1
 	};
 
@@ -2237,9 +2245,9 @@ int ctx_by_name(struct cfg_ctx *ctx, const char *id, checks check)
 		for_each_connection(conn, &res->connections) {
 			if (hi) { /* it was host name */
 				if (res->me == hi) {
-					err("Host name '%s' (given with --peer option) is not a "
+					err("Host name '%s' (from '%s') is not a "
 					    "peer, but the local node\n",
-					    conn_or_hostname);
+					    conn_or_hostname, id);
 					return -ENOENT;
 				}
 
@@ -3077,9 +3085,6 @@ int parse_options(int argc, char **argv, struct adm_cmd **cmd, char ***resource_
 			print_platform_specific_versions();
 			exit(0);
 			break;
-		case 'P':
-			connect_to_host = optarg;
-			break;
 		case 'W':
 			insert_tail(&backend_options, names_from_str(optarg));
 			break;
@@ -3302,6 +3307,11 @@ int main(int argc, char **argv)
 	int is_adjust;
 	struct cfg_ctx ctx = { };
 
+	if (argv == NULL || argc < 1) {
+		fputs("drbdadm: Nonexistent or empty arguments array, aborting.\n", stderr);
+		abort();
+	}
+
 	initialize_err();
 	initialize_deferred_cmds();
 	yyin = NULL;
