@@ -331,7 +331,7 @@ void set_me_in_resource(struct d_resource* res, int match_on_proxy)
 			    host->lower ? "stacked-on-top-of" : "on",
 			    host->lower ? host->lower->name : names_to_str(&host->on_hosts));
 		}
-		if (res->me && res->me != host) {
+		if (res->me && res->me != host && !match_on_proxy) {
 			config_valid = 0;
 			err("%s:%d: in resource %s, %s %s { ... } ... %s %s { ... }:\n"
 			    "\tThere are multiple host sections for this node.\n",
@@ -360,7 +360,7 @@ void set_me_in_resource(struct d_resource* res, int match_on_proxy)
 
 		for_each_path(path, &conn->paths) {
 			STAILQ_FOREACH(h, &path->hname_address_pairs, link) {
-				if (h->host_info == res->me)
+				if (h->host_info && h->host_info->used_as_me)
 					break;
 			}
 
@@ -368,7 +368,9 @@ void set_me_in_resource(struct d_resource* res, int match_on_proxy)
 				/* In case this is a path of a stacked resource ignore it if
 				   the path belongs to the "non local" node of the lower resource.
 				   Never ignore an implicit path, since they have faked hostnames */
-				if (!h->faked_hostname && h->host_info->lower &&
+				if (!h->faked_hostname &&
+				    h->host_info->lower &&
+				    h->host_info->lower->me &&
 				    !hostname_in_list(h->name, &h->host_info->lower->me->on_hosts))
 					path->ignore = 1;
 
