@@ -34,6 +34,8 @@
 #include "../drbdsetup.h"
 #include "../drbdsetup_colors.h"
 
+#include "drbd_protocol.h"
+
 int print_event(struct drbd_cmd *cm, struct genl_info *info, void *u_ptr);
 
 extern struct genl_family drbd_genl_family;
@@ -45,9 +47,17 @@ static __u32 test_volume_number = 0;
 static __u32 test_volume_number_b = 1;
 static __u32 test_peer_node_id = 1;
 static char *test_peer_name = "some-peer";
+static char *test_backing_dev_path = "/dev/sda";
 
 typedef void (*test_msg)(struct msg_buff *smsg);
 
+static char *backing_dev(__u32 disk_state)
+{
+	if (disk_state == D_UP_TO_DATE || disk_state == D_INCONSISTENT)
+		return test_backing_dev_path;
+	return "none";
+}
+
 void test_msg_put(struct msg_buff *smsg, __u8 cmd, __u32 minor)
 {
 	struct drbd_genlmsghdr *dhdr;
@@ -104,6 +114,7 @@ void test_device_info(struct msg_buff *smsg, __u32 dev_disk_state, __u8 dev_has_
 	nla_put_u32(smsg, T_dev_disk_state, dev_disk_state);
 	nla_put_u8(smsg, T_is_intentional_diskless, false);
 	nla_put_u8(smsg, T_dev_has_quorum, dev_has_quorum);
+	nla_put_string(smsg, T_backing_dev_path, backing_dev(dev_disk_state));
 	nla_nest_end(smsg, nla);
 }
 
@@ -196,6 +207,7 @@ void test_peer_device_statistics(struct msg_buff *smsg, bool resync)
 	nla_put_u64(smsg, T_peer_dev_rs_dt1_ms, resync ? 100 : 0);
 	nla_put_u64(smsg, T_peer_dev_rs_db1_sectors, resync ? 50 : 0);
 	nla_put_u32(smsg, T_peer_dev_rs_c_sync_rate, 0);
+	nla_put_u64(smsg, T_peer_dev_uuid_flags, UUID_FLAG_STABLE);
 	nla_nest_end(smsg, nla);
 }
 
@@ -461,6 +473,7 @@ void test_helper_response(struct msg_buff *smsg)
 
 int test_print_event(struct nlmsghdr *nlh)
 {
+	struct nlattr *tla[128];
 	struct drbd_cmd cm;
 	int err;
 	/* read message as if receiving */
@@ -469,9 +482,9 @@ int test_print_event(struct nlmsghdr *nlh)
 		.nlhdr = nlh,
 		.genlhdr = nlmsg_data(nlh),
 		.userhdr = genlmsg_data(nlmsg_data(nlh)),
-		.attrs = global_attrs,
+		.attrs = tla,
 	};
-	err = drbd_tla_parse(nlh);
+	err = drbd_tla_parse(tla, nlh);
 	if (err) {
 		fprintf(stderr, "drbd_tla_parse() failed");
 		return 1;
@@ -574,6 +587,8 @@ int main(int argc, char **argv)
 		{ "timestamps", no_argument, 0, 'T' },
 		{ "statistics", no_argument, 0, 's' },
 		{ "now", no_argument, 0, 'n' },
+		{ "diff", optional_argument, 0, 'i' },
+		{ "full", no_argument, 0, 'f' },
 		{ "color", no_argument, 0, 'c' },
 		{ }
 	};
@@ -581,7 +596,7 @@ int main(int argc, char **argv)
 	opt_color = NEVER_COLOR;
 	for(;;) {
 		int c;
-		c = getopt_long(argc, argv, "hTsnc::", options, NULL);
+		c = getopt_long(argc, argv, "hTifsnc::", options, NULL);
 		if (c == -1)
 			break;
 		switch(c) {
@@ -590,7 +605,7 @@ int main(int argc, char **argv)
 		case '?':
 			fprintf(stderr, "drbdsetup_events2_instrumented - Fake drbdsetup events2 from messages on stdin\n\n");
 			fprintf(stderr, "USAGE: %s [options]\n", argv[0]);
-			fprintf(stderr, "    [--timestamps] [--statistics] [--now] [--color]\n");
+			fprintf(stderr, "    [--timestamps] [--statistics] [--now] [--diff] [--full] [--color]\n");
 			return 1;
 
 		case 'n':
@@ -606,6 +621,16 @@ int main(int argc, char **argv)
 			opt_timestamps = true;
 			break;
 
+		case 'i':
+			opt_diff = true;
+			break;
+
+		case 'f':
+			++opt_verbose;
+			opt_statistics = true;
+			opt_fullch = true;
+			break;
+
 		case 'c':
 			opt_color = ALWAYS_COLOR;
 			break;
